//
// Copyright (c) 2012 Rodolphe Breard
// 
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//

var NsClient = function() {
    this.storage = chrome.storage.sync;
    this.state = "actif";
    this.allowed_statuses =  ["actif", "away", "idle", "lock"];
    this.is_connected = false;
    this.socket = null;
};

NsClient.prototype.connect = function() {
    var cnt = function(elem) {
	return function(infos) {
	    if (typeof infos.login !== "undefined" && typeof infos.pwd_socks !== "undefined") {
		chrome.socket.create('tcp', {}, function(sock_inf) {
		    elem.socket = sock_inf.socketId;
		    chrome.socket.connect(elem.socket, "ns-server.epita.fr", 4242, function(res) {
			chrome.socket.read(elem.socket, null, function(rd_inf) {
			    if (rd_inf.resultCode > 0) {
				var data = ab2str(rd_inf.data).split(' '),
				auth = "ext_user_log ";
				auth += infos.login + " ";
				auth += hex_md5(data[2] + "-" + data[3] + "/" + data[4] + infos.pwd_socks) + " ";
				auth += "chromesoul chromesoul\n";
				chrome.socket.write(elem.socket, str2ab("auth_ag ext_user none none\n"), function(w_inf) {
				    chrome.socket.read(elem.socket, null, function(rd_inf) {
					if (rd_inf.resultCode > 0) {
					    chrome.socket.write(elem.socket, str2ab(auth), function(w_inf) {
						chrome.socket.read(elem.socket, null, function(rd_inf) {
						    if (rd_inf.resultCode > 0) {
							elem.is_connected = true;
							console.info("connected to the netsoul server");
							elem.updateStatus();
							elem.daemonize();
						    }
						});
					    });
					}
				    });
				});
			    }
			});
		    });
		});
	    }
	};
    };

    if (!this.is_connected) {
	this.storage.get(null, cnt(this));
    }
};

NsClient.prototype.disconnect = function() {
    if (this.is_connected) {
	chrome.socket.disconnect(this.socket);
	this.is_connected = false;
	console.info("disconnected");
	this.updateStatus();
    }
};

NsClient.prototype.daemonize = function() {
    var dm = function(elem) {
	return function(rd_inf) {
	    if (rd_inf.resultCode > 0) {
		var data = ab2str(rd_inf.data);

		if (data.substr(0, 5) === "ping ") {
		    chrome.socket.write(elem.socket, rd_inf.data, function(w_inf) {
			elem.daemonize();
		    });
		} else {
		    elem.daemonize();
		}
	    } else {
		/*
		 * A read error is the only way to know if the remote peer has disconnected.
		 * See <http://developer.chrome.com/apps/socket.html> for more informations.
		 */
		elem.disconnect();
		console.info('connection lost, reconnecting...');
	    }
	};
    };

    if (this.is_connected) {
	chrome.socket.read(this.socket, dm(this));
    } else {
	console.error("unable to daemonize: not connected");
    }
};

NsClient.prototype.updateStatus = function() {
    var status_txt = document.getElementById("status_txt");

    if (this.is_connected) {
	if (status_txt !== null)
	    status_txt.innerHTML = this.state;

	var status_msg = "user_cmd state ";
	status_msg += this.state + ":";
	status_msg += Math.round(new Date().getTime() / 1000) + "\n";
	chrome.socket.write(this.socket, str2ab(status_msg), function(w_inf) {});
    } else {
	if (status_txt !== null)
	    status_txt.innerHTML = 'disconnected';
    }
};

NsClient.prototype.changeStatus = function(new_status) {
    if (this.is_connected) {
	if (this.allowed_statuses.indexOf(new_status) !== -1) {
	    this.status = new_status;
	} else {
	    console.warn("invalid status: " + new_status);
	}
    } else {
	console.warn("not connected");
    }
};

NsClient.prototype.init = function() {
    var status_update = function(elem) {
	return function() {
	    elem.updateStatus();
	};
    },
    connect = function(elem) {
	return function() {
	    if (!elem.is_connected) {
		elem.connect();
	    }
	};
    };

    status_update(this).apply();
    connect(this).apply();
    setInterval(connect(this), 10000);
    setInterval(status_update(this), 600000);
};
